/* 
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

var sequencer = {   // GLOBAL VAR TO STORE SEQUENCER SETTINGS!
    uid: "",                // The unique id of this sequencer (not DB related!). This will help to avoid global var conflicts. Assign this randomly! (STRING)
    session: {              // Properties of the currently loaded session
        title: "",              // Title or name (STRING)
        uid: "",                // Database UID of the current session (STRING)
        pipeline: {             // Pipeline
            uids: [],               // Uids of objects in pipeline (STRING)
            types: [],              // Types of objects in pipeline (STRING)
            upToDate: []            // Whether or not object displays are up to date (BOOL)
        }
    },
    state: {                // Operating state of the sequencer
        numSteps: 0,            // Number of steps currently drawn in the editor (not necessarily same as number of steps in pipeline!) (INTEGER)
        loaded: false,          // Whether or not the sequencer content has been updated for the first time (BOOL)
        selectedStep: {         // Properties of the currently selected step
            uid: "",                // UID of this step (STRING)
            index: null               // The 'index' of this step in the current sequencer view (NOT the pipeline!) (INTEGER)
        }        
    },
    settings: {             // Various settings to determine the workings of the sequencer
        content: {              // Properties related to the content view of the sequencer
            width: null,            // Width of the viewing area (INTEGER)
            height: null,           // Height of the viewing area (INTEGER)
            maxObjects: null,       // The maximum number of content elements to be displayed at once time (INTEGER)
            orientation: "horizontal"        // Whether the editor should be a vertical or horizontal editor (STRING)
        },
        efficientUpdating: true // Whether or not to use selective querying of the database for new step objects. True will only refresh out-of-date steps, False will refresh the whole pipeline
    }
}


function SubmitToolbox(type) {
    deselectStep();
    var c = "objectToCreate="+type;
    var u = "createObject.php";
        
    newAjaxRequest(c, u, function(result) {
        sequencer.session.pipeline.uids.push(removeNL(result.responseText));
        sequencer.session.pipeline.types.push(type);
        sequencer.session.pipeline.upToDate.push(false);
        updateSequencer();
    }, true);
}

function clickStep(uid) {
    if (uid == sequencer.state.selectedStep.uid) {  // user deselected a currently selected step.
        deselectStep();
    }
    else {
        if (sequencer.state.selectedStep.uid != undefined && sequencer.state.selectedStep.uid != "") {
            deselectStep();
            selectStep(uid);
        }
        else {
            selectStep(uid);
        }        
    }
}

function selectStep(uid) {
    var element = document.getElementById(uid);
    if (element) {
        addClass(element, "selected");
        var type = sequencer.session.pipeline.types[sequencer.session.pipeline.uids.indexOf(uid)];
        ajaxInfoRequest(uid, document.getElementById("infoPanelContent"), type);
        sequencer.state.selectedStep.uid = uid;
        sequencer.state.selectedStep.index = null;  //Don't know how to do this yet.
    }
}

function deleteStep() {
    // check if a step is selected
    if (sequencer.state.selectedStep.uid == null) {
        return;
    }
    var uid = sequencer.state.selectedStep.uid;
    // deselect the step to reset the info panel and selection code
    deselectStep();
    // splice the step's data from the pipeline
    var index = sequencer.session.pipeline.uids.indexOf(uid);
    if (index >= 0 && index < sequencer.session.pipeline.uids.length) {
        sequencer.session.pipeline.uids.splice(index, 1);
        sequencer.session.pipeline.types.splice(index, 1);
        sequencer.session.pipeline.upToDate.splice(index, 1);
        // Then delete the step visually
        var element = document.getElementById(uid);
        var divider;
        if (!element.nextSibling) {
            // the element is at the end of the pipeline
            // therefore we remove the previous divider.
            // Note: it could also be the only element left in the pipeline!
            divider = (element.previousSibling) ? element.previousSibling : false;
            if (divider != false) {
                divider.parentNode.removeChild(divider);
            }
        }
        else {
            // the element is at any position except the last, therefore we remove the next divider
            divider = (element.nextSibling) ? element.nextSibling : false;
            if (divider != false) {
                divider.parentNode.removeChild(divider);
            }
        }
        
        // Finally, remove the element itself.
        element.parentNode.removeChild(element);
        sequencer.state.numSteps--;
        
    }
}

function deselectStep() {
    if (!sequencer.state.selectedStep.uid) return;
    var element = document.getElementById(sequencer.state.selectedStep.uid);
    removeClass(element, "selected");
    sequencer.state.selectedStep.uid = null;
    sequencer.state.selectedStep.index = null;
    var infoPanel = document.getElementById("infoPanelContent");
    while (infoPanel.firstChild) infoPanel.removeChild(infoPanel.firstChild);
}

function ajaxInfoRequest(uid, el, type) {
    var c = "uid="+uid;
    c += "&type="+type;
    var u = "getInfo.php";
    newAjaxRequest(c, u, function(result) {
        el.innerHTML = result.responseText;
    }, true);
}

function updateSequencer() {

    /*
 * Description:
 * This function updates the visual elements in the sequencer content view to match the current state of the sequencer.session.pipeline property.
 * It queries the database for object properties via AJAX (returnStep/Display/.php), then inserts divider div's in between where needed.
 */
    var content = document.getElementById("seqContentWrapper");
    if (sequencer.state.loaded == false || sequencer.settings.efficientUpdating == false) {  // This is the first update of the sequencer since page load, therefore it contains no previous steps
        // First clear the entire content wrapper, just for safety and in case efficientUpdating is off
        while (content.firstChild) {
            content.removeChild(content.firstChild);
        }
        // Compose a request string - "uids=1,2,3&types=a,b,c"
        var requestString = "uids=";
        for (var i = 0; i < sequencer.session.pipeline.uids.length; i++) {
            requestString += sequencer.session.pipeline.uids[i]+",";
            sequencer.state.numSteps++;
        }
        
        requestString = requestString.slice(0, requestString.length - 1);   // remove trailing commas
        requestString += "&types=";
        for (var j = 0; j < sequencer.session.pipeline.types.length; j++) {
            requestString += sequencer.session.pipeline.types[j]+",";
        }
        requestString = requestString.slice(0, requestString.length - 1);   // remove trailing commas
        newAjaxRequest(requestString, "returnStep.php", function(result) {
            content.innerHTML = result.responseText;
            updateDividers();
            sequencer.state.loaded = true;
            for (var i = 0; i < sequencer.session.pipeline.upToDate.length; i++) {
                sequencer.session.pipeline.upToDate.length[i] = true;
            }
        }, true);
    }
    else {
        // Pack together all needed updates in one request string, insert responseText.firstChild whenever sequencer.session.pipeline.upToDate[i] == false
        debugger;
        var needsUpdating = new Array();
        
        // DIT IS FOUT.
        // BASIC IDEE, IK LOOP NU DOOR DE CHILNODES HEEN, EN NIET DOOR DE PIPELINE ARRAY. HIERDOOR KAN JE NOOIT EEN NIEUWE OBJECT DRAWEN, WANT DAAR IS IN DE SEQUENCER NOG GEEN DIV VOOR.
        // OPLOSSING: 
        // LOOP DOOR SEQUENCER EN HAAL ALLES WEG DAT ER NIET HOORT (CROSSCHECK DISPLAYSTEP UID MET ARRAY)
        // LOOP DAARNA DOOR DE PIPELINE EDITOR, CHECK OF ER ELEMENTS VOOR ZIJN. ZO NIET, VOEG TOE AAN DE EDITOR!
        
        for (var i = 0; i < sequencer.session.pipeline.uids.length; i++) {
            
        }
        
        for (var i = 0; i < content.childNodes.length; i++) {
            // If this childNode is not a displayStep, continue to the next childNode
            if (!hasClass(content.childNodes[i], "displayStep")) continue;
            // If this step is up to date, continue
            // pseudo: the upToDate entry on the index where childNodes[i].id is stored in uids
            if (sequencer.session.pipeline.upToDate[sequencer.session.pipeline.uids.indexOf(content.childNodes[i].id)] == true) continue;
            
            // So now we know that we are looking at a displayStep and it needs to be refreshed.
            // We store the current childNodes index, the uid of the proper displayStep and the type of the proper displayStep in needsUpdating array.
            needsUpdating.push(new Array(i, sequencer.session.pipeline.uids[i], sequencer.session.pipeline.types[i]));
        }
        
        // Now that we have an array of the steps that need to be refreshed, we compose a requestString.
        requestString = "uids=";
        for (var i = 0; i < needsUpdating.length; i++) {
            requestString += needsUpdating[i][1]+",";   // comma separated, of course
        }
        requestString = requestString.slice(0, requestString.length - 1);   // remove trailing commas
        requestString += "&types=";
        for (var i = 0; i< needsUpdating.length; i++) {
            requestString += needsUpdating[i][2]+",";
        }
        requestString = requestString.slice(0, requestString.length - 1);   // remove trailing commas
        
        // Now it is time to send this string to returnStep.php (Soon to be returnStepDisplay.php!).
        
        alert(requestString);
        newAjaxRequest(requestString, "returnStep.php", function(result) {
            //alert(result.responseText);
            insertNewSteps(responseText);
        }, true);
    }
    
}

function loadSequencer() {
    /* 
     * Description:
     * Load data from hidden fields (put there by PHP), store them in the global var "sequencer" (as well as several initialization properties),
     * then remove the hidden fields from the HTML document tree.
     */
    
    // Load hidden fields and set required properties in global object var.
    try {
        
        var fPipelineString = document.getElementById("pipelineStringField");
        var fPipelineTypes = document.getElementById("pipelineTypeField");
        var fSessionUid = document.getElementById("sessionField");
        var fSessionTitle = document.getElementById("sessionTitleField");
    
        sequencer.session.title = fSessionTitle.value;
        sequencer.session.uid = fSessionUid.value;
        sequencer.session.pipeline.uids = stringToArray(fPipelineString.value, ",");
        sequencer.session.pipeline.types = stringToArray(fPipelineTypes.value, ",");
        sequencer.session.pipeline.upToDate = new Array();
        
        for (var i = 0; i < sequencer.session.pipeline.uids.length; i++) {
            sequencer.session.pipeline.upToDate.push(true);
        }
        sequencer.state.numSteps = 0;
        sequencer.state.loaded = false;
        sequencer.settings.content.orientation = "h";
        sequencer.settings.efficientUpdating = true;
    }
    catch (e) {
        // Alert developer of any errors while setting these variables
        for (error in e) alert(error.message);
    }
    
    // Then remove the hidden fields from the HTML document
    var hiddenInputs = document.getElementById("hiddenInputs");
    hiddenInputs.parentNode.removeChild(hiddenInputs);
    
    console.log(sequencer);
    // finally, run updateSequencer to refresh the visual display of the pipeline
    
    updateSequencer();
}

function insertNewSteps(responseText) {
    debugger;
    var content = document.getElementById("seqContentWrapper");
    // Create a holder Div that converts the returned HTML to HTMLDivElements implicitly, so that they can be manipulated using DOM methods
    var holderDiv = document.createElement("div");
    holderDiv.innerHTML = responseText;
    // tempDiv will serve to 
    var tempDiv;
    // Loop through 
    for (var i = 0; i < content.childNodes.length; i++) {
        // Once again, skip if not a displayStep or the given displayStep is already up to date.
        if (!hasClass(content.childNodes[i], "displayStep")) continue;
        if (sequencer.session.pipeline.upToDate[sequencer.session.pipeline.uids.indexOf(content.childNodes[i].id)] == true) continue;
        tempDiv = holderDiv.firstChild;
        holderDiv.removeChild(holderDiv.firstChild);
        content.replaceChild(tempDiv, content.childNodes[i]);
        // Exit the loop if there is no more responsetext
        if (!holderDiv.firstChild) break;
    }
}

function updateDividers() {
    var content = document.getElementById("seqContentWrapper");
    for (var i = 0; i < content.childNodes.length; i++) {
        var element = content.childNodes[i];
        if (!hasClass(element, "displayStep")) {
            continue;
        }
        var lastElement = (element.previousSibling && element.previousSibling.nodeName == "DIV") ? element.previousSibling : false;
        var nextElement = (element.nextSibling && element.nextSibling.nodeName == "DIV") ? element.nextSibling : false;
        if (lastElement != false) {
            if (!hasClass(lastElement, "divider")){
                var newDivider = document.createElement("div");
                addClass(newDivider, "divider");
                addClass(newDivider, sequencer.settings.content.orientation);
                content.insertBefore(newDivider, element);
                delete newDivider;
            }
        }
        if (nextElement != false) {
            if (!hasClass(nextElement, "divider")){
                var newDivider = document.createElement("div");
                addClass(newDivider, "divider");
                addClass(newDivider, sequencer.settings.content.orientation);
                content.insertBefore(newDivider, nextElement);
                delete newDivider;
            }
        }
    }
}

function savePipeline (confirmSave) {
    if (confirmSave==true) {
        var answer = confirm("Save changes to pipeline?");
    }
    else {
        var answer = true;
    }
    
    
    if (answer) {
        var pipeline = document.getElementById("pipelineStringField").value;
        pipeline = pipeline.slice(0, pipeline.length - 1);
        var types = document.getElementById("pipelineTypeField").value;
        types = types.slice(0, types.length - 1);
        var session = document.getElementById("sessionField").value;
        var requestString = "uids="+pipeline+"&types="+types+"&sessionUid="+session;
        console.log(requestString);
        
        
        var success;
        newAjaxRequest(requestString, "savesession.php", function(result){
            success = result.responseText;
        }, true);
        console.log(success);
    }
}

function editStep() {
    // first save
    
    //savePipeline(false);
    
    var postForm = document.createElement("form");
    var type = sequencer.session.pipeline.types[sequencer.session.pipeline.uids.indexOf(sequencer.state.selectedStep.uid)];
    postForm.action = type.toLowerCase()+"Editor.php";
    postForm.method = "POST";
    var objectUid = document.createElement("input");
    objectUid.type = "hidden";
    objectUid.name = "objectUid";
    objectUid.value = sequencer.state.selectedStep.uid;
    postForm.appendChild(objectUid);
    postForm.submit();
}

function moveStep (direction) {
    // Check if a step is selected
    
    if (sequencer.state.selectedStep.uid == null || direction == null) return;
    // Check if the step is not at either end of the pipeline
    var index = sequencer.session.pipeline.uids.indexOf(sequencer.state.selectedStep.uid);
    if ((index < 0) || (index >= sequencer.session.pipeline.uids.length) || (index == 0 && direction == -1) || (index == sequencer.session.pipeline.uids.length - 1 && direction == 1)) {
        alert("Cannot move out of bounds!");
        return;
    }
    
    // Find the two elements in the editor content display
    var content = document.getElementById("seqContentWrapper");
    var element = document.getElementById(sequencer.session.pipeline.uids[index]);
    var otherElement = document.getElementById(sequencer.session.pipeline.uids[index+direction]);
    // First buffer the two elements
    var tempElement = element.cloneNode(true);
    var tempOtherElement = otherElement.cloneNode(true);
    var placeHolderElement = document.createElement("div");
    placeHolderElement.id = "placeholder_element";
    content.replaceChild(placeHolderElement, otherElement);
    content.replaceChild(tempOtherElement, element);
    content.replaceChild(tempElement, placeHolderElement);
    //This should work.
    // A-B     Start positions, backup to tA and tB
    // A-X     Replace B with placeholder X
    // B-X     Replace A with tB
    // B-A     Replace placeholder X with tA.
    // The two elements are now swapped.
     
    // Now swap the array entries.
    sequencer.session.pipeline.uids[index] = sequencer.session.pipeline.uids.splice(index+direction, 1, sequencer.session.pipeline.uids[index])[0];
    sequencer.session.pipeline.types[index] = sequencer.session.pipeline.types.splice(index+direction, 1, sequencer.session.pipeline.types[index])[0];
     
}